package gov.va.med.vistalink.management;

import gov.va.med.exception.ExceptionUtils;
import gov.va.med.vistalink.adapter.cci.VistaLinkResourceException;
import gov.va.med.vistalink.adapter.config.Connector;
import gov.va.med.vistalink.adapter.config.ConnectorsDocument;
import gov.va.med.vistalink.adapter.config.StationNumbers;
import gov.va.med.crypto.DESPassPhraseEncrypter;
import gov.va.med.crypto.EncrypterRegistry;
import gov.va.med.vistalink.adapter.spi.ConfigurationReader;

import java.io.File;
import java.io.IOException;

import java.util.Iterator;
import java.util.Map;
import java.util.Set;
import java.util.Calendar;
import java.util.Date;
import java.util.HashSet;

import java.net.URL;

import javax.management.ObjectName;
import javax.management.QueryExp;

import javax.naming.InitialContext;
import javax.naming.NamingException;

import javax.servlet.http.HttpServletRequest;

import org.apache.log4j.Logger;
import org.apache.xmlbeans.XmlException;
import org.apache.xmlbeans.XmlObject;

import weblogic.management.MBeanHome;
import weblogic.management.RemoteMBeanServer;

import gov.va.med.vistalink.management.ui.tree.*;

/**
 * XML configuration file worker.
 * 
 * @author Application Modernization - Foundations Team
 * @version 1.5.0.026
 */
public class ConnectorConfigurator implements java.io.Serializable{
  private static final String FILE_PATH_PROPERTY = "getConfigFilePath";

  public static final int OPERATIONAL_MODE_PATH = 0;

  public static final int OPERATIONAL_MODE_STREAM = 1;

  public static final char TREE_DELIMITER = '|';

  public static final String TREE_NAME = "localTree";

  public static final String EMPTY_FILE = "<?xml version=\"1.0\" encoding=\"UTF-8\"?>"
      + "<connectors xsi:noNamespaceSchemaLocation=\"connectorConfig.xsd\" xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\" xmlns=\"http://med.va.gov/vistalink/adapter/config\">"
      + "</connectors>";

  private int oMode = OPERATIONAL_MODE_PATH;

  private String filePath = null;

  private ConnectorsDocument configFile;

  public String sFile; //for now

  private String serverName = null;

  private boolean isAutoSave = true;

  private int dirtyCount = 0;

  private static final Logger logger = Logger
      .getLogger(ConnectorConfigurator.class);

  protected String getConnectorProperty(String propName){
    QueryExp query = null;
    StringBuffer sb = new StringBuffer();

    try {
      if( serverName == null )
        throw new RuntimeException("serverName must be set!");
      sb.append("\nTrying to get MBean home...");
      MBeanHome home = getMBeanHome(serverName);
      sb.append("\nGot MBean Home: " + home.toString());
      sb.append("\nTrying to get domainName...");
      String domainName = home.getDomainName();
      sb.append("\nGot domainName: " + domainName);
      sb.append("\nTrying to get MBeanServer...");
      RemoteMBeanServer mbeanServer = home.getMBeanServer();
      sb.append("\nGot MBeanServer: " + mbeanServer.toString());
      Set cfBeans = mbeanServer.queryNames(new ObjectName(domainName
          + ":Type=ConnectionFactoryMonitor,*"), query);
      sb.append("\nGot a Set of Beans:");
      ObjectName cfBean = null;
      // Iterator through, get output
      for( Iterator iter = cfBeans.iterator(); iter.hasNext(); ) {
        cfBean = (ObjectName) iter.next();
        break;
      }
      return (String) mbeanServer.invoke(cfBean, propName, new Object[0],
          new String[0]);
    } catch (Exception e) {
      logger.error("Could not get Connector properties Information: "
          + e.getMessage() + "\n" + ExceptionUtils.getFullStackTrace(e));
      throw new RuntimeException(e.getMessage() + sb.toString());
    }
  }

  protected static MBeanHome getMBeanHome(String sName){
    MBeanHome home = null;
    try {
      InitialContext ctx = new InitialContext();
      home = (MBeanHome) ctx.lookup(MBeanHome.JNDI_NAME + "." + sName);
    } catch (NamingException e) {
      logger.error("Could not locate MBean Home for server '" + sName + "/n"
          + e.getMessage());
      throw new RuntimeException("Could not locate MBean Home for server '"
          + sName + "'./n Check to see if this server has been started.");
    }
    return home;
  }

  public static void getConnectorProps(Map props, String sName){
    String[] PROPS = { "getConnectionFactoryName", "getEISType",
        "getHostIpAddress", "getJNDIName", "getName", "getPoolName",
        "getResourceAdapterLinkRef", "getStationNumbers", "getSystemInfo",
        "getConfigFilePath" };
    QueryExp query = null;

    try {
      MBeanHome home = getMBeanHome(sName);
      String domainName = home.getDomainName();
      RemoteMBeanServer mbeanServer = home.getMBeanServer();
      Set cfBeans = mbeanServer.queryNames(new ObjectName(domainName
          + ":Type=ConnectionFactoryMonitor,*"), query);

      // Iterator through, get output
      for( Iterator iter = cfBeans.iterator(); iter.hasNext(); ) {
        ObjectName cfBean = (ObjectName) iter.next();
        for( int inx = 0; inx < PROPS.length; inx++ )
          props.put(PROPS[inx], mbeanServer.invoke(cfBean, PROPS[inx],
              new Object[0], new String[0]));
      }
    } catch (Exception e) {
      logger.error("Could not get Connector properties Information: "
          + e.getMessage() + "\n" + ExceptionUtils.getFullStackTrace(e));
      throw new RuntimeException(e);
    }
  }

  public String getFilePath() throws VistaLinkResourceException{
    //if( filePath == null )
    //  filePath = getConnectorProperty(FILE_PATH_PROPERTY);
    if( oMode == OPERATIONAL_MODE_PATH ) {
      if( filePath == null ) {
        try {
          URL configURL = this.getClass().getResource(
              ConfigurationReader.CONFIG_FILE_PATH);
          filePath = configURL == null?null:configURL.getPath();
        } catch (Exception e) {
          String error = "Problem accessing config file. Make sure it resides on the CLASSPATH. ";
          logger.error(error, e);
          throw new VistaLinkResourceException(error, e);
        }
      }
    }
    return filePath;
  }

  public boolean isFileAssigned(){
    boolean result = false;
    if( oMode == OPERATIONAL_MODE_PATH ) {
      try {
        getFilePath();
      } catch (Exception e) {

      }
      result = filePath != null;
    } else if( oMode == OPERATIONAL_MODE_STREAM )
      result = sFile != null;
    return result;
  }

  protected void readFile() throws VistaLinkResourceException{
    if( logger.isDebugEnabled() )
      logger.debug("Reading Config File, Path: " + getFilePath());

    if( getFilePath() == null ) {
      String error = "Missing configuration file path.";
      logger.error(error);
      throw new VistaLinkResourceException(error);
    }

    try {
      XmlObject xo;
      if( oMode == OPERATIONAL_MODE_PATH ) {
        File listenerConfig = new File(getFilePath());
        xo = ConnectorsDocument.Factory.parse(listenerConfig);
      } else
        xo = ConnectorsDocument.Factory.parse(sFile);

      if( xo instanceof ConnectorsDocument ) {
        HashSet errors = new HashSet();
        org.apache.xmlbeans.XmlOptions vOptions = new org.apache.xmlbeans.XmlOptions();
        vOptions.setErrorListener(errors);
        if( !xo.validate(vOptions) )
          throw new VistaLinkResourceException(
              "Not a valid Connectors document.\n" + errors.toString() + "\n");
        configFile = (ConnectorsDocument) xo;
      } else {
        throw new VistaLinkResourceException("Not a \"Connectors\" document");
      }

    } catch (XmlException e) {
      String error = "Problem parsing config file. ";
      logger.error(error, e);
      throw new VistaLinkResourceException(error, e);
    } catch (IOException e) {
      String error = "Problem opening config file.";
      logger.error(error, e);
      throw new VistaLinkResourceException(error, e);
    }
  }

  public ConnectorConfigurator(){
  }

  // =======================================
  public boolean isFileRead(){
    return configFile != null;
  }

  public ConnectorsDocument.Connectors getConnectors()
      throws VistaLinkResourceException{
    if( !isFileRead() )
      readFile();

    return configFile.getConnectors();
  }

  public Connector[] getConnectorArray() throws VistaLinkResourceException{
    if( !isFileRead() )
      readFile();
    return configFile.getConnectors().getConnectorArray();

  }

  public Connector getConnectorByName(String lName)
      throws VistaLinkResourceException{
    Connector result = null;
    Connector[] ls = getConnectorArray();
    for( int inx = 0; inx < ls.length; inx++ ) {
      result = ls[inx];
      if( result.getJndiName().equals(lName) )
        break;
      result = null;
    }
    return result;
  }

  public void refresh(){
    configFile = null;
  }

  public boolean save() throws VistaLinkResourceException{
    boolean result = false;
    HashSet errors = new HashSet();
    if( validateFile(errors) ) {
      if( oMode == OPERATIONAL_MODE_PATH )
        try {
          configFile.save(new File(getFilePath()));
          dirtyCount = 0;
          result = true;
        } catch (Exception e) {
          throw new VistaLinkResourceException(
              "Error saving configuration file: " + e.getMessage());
        }
    } else
      throw new VistaLinkResourceException(
          "Error validating configuration file: " + errors.toString());
    return result;
  }

  public Connector addConnector(String lName) throws VistaLinkResourceException{
    gov.va.med.vistalink.adapter.config.Connector result = null;
    if( isFileRead() ) {
      if( getConnectorByName(lName) != null )
        throw new VistaLinkResourceException("Duplicate Connector Name!");
      result = configFile.getConnectors().addNewConnector();
      result.setJndiName(lName);
      result.setAlwaysUseDefaultAsMin(true);
      ++dirtyCount;
    }
    return result;
  }

  public void deleteConnector(String lName) throws VistaLinkResourceException{
    if( isFileRead() ) {
      Connector[] la = getConnectorArray();
      for( int inx = 0; inx < la.length; inx++ )
        if( lName.equals(la[inx].getJndiName()) ) {
          configFile.getConnectors().removeConnector(inx);
          ++dirtyCount;
          if( isAutoSave )
            save();
          break;
        }
    }
  }

  public String getConnectorValue(String lName, String pName){
    Connector l = null;
    String result = null;
    try {
      l = getConnectorByName(lName);

      if( ConnectorConfiguratorConstants.PROPERTY_ACCESSCODE.NAME.equals(pName) )
        if( l.getEncrypted() )
          result = getDecryptedValue(l.getAccessCode());
        else
          result = l.getAccessCode();
      else if( ConnectorConfiguratorConstants.PROPERTY_IP.NAME.equals(pName) )
        result = String.valueOf(l.getIp());
      else if( ConnectorConfiguratorConstants.PROPERTY_VERIFYCODE.NAME.equals(pName) )
        if( l.getEncrypted() )
          result = getDecryptedValue(l.getVerifyCode());
        else
          result = l.getVerifyCode();
      else if( ConnectorConfiguratorConstants.PROPERTY_PORT.NAME.equals(pName) )
        result = String.valueOf(l.getPort());
      else if( ConnectorConfiguratorConstants.PROPERTY_PSN.NAME.equals(pName) )
        result = String.valueOf(l.getPrimaryStation());
      else if( ConnectorConfiguratorConstants.PROPERTY_ISENABLED.NAME.equals(pName) )
        result = String.valueOf(l.getEnabled());
      else if( ConnectorConfiguratorConstants.PROPERTY_TIMESTAMP.NAME.equals(pName) )
        result = l.getTimestamp().getTime().toString();
      else if( ConnectorConfiguratorConstants.PROPERTY_ISTIMEOUT.NAME.equals(pName) )
        result = String.valueOf(l.getAlwaysUseDefaultAsMin());
      else if( ConnectorConfiguratorConstants.PROPERTY_TIMEOUT.NAME.equals(pName) )
        result = String.valueOf(l.getTimeout());
    } catch (Exception e) {
      result = e.getMessage();
    }
    return result;
  }

  public boolean validateFile(java.util.Collection errors){
    boolean result = isFileRead();
    if( result ) {
      if( errors != null ) {
        org.apache.xmlbeans.XmlOptions vOptions = new org.apache.xmlbeans.XmlOptions();
        vOptions.setErrorListener(errors);
        result = configFile.validate(vOptions);
      } else
        result = configFile.validate();
    }
    return result;
  }

  public void setFileAsString(String fs, String fp){
    if( fs != null ) {
      sFile = fs;
      configFile = null;
      filePath = fp;
      oMode = OPERATIONAL_MODE_STREAM;
    }
  }

  public String getFileAsString(){
    String result = null;
    if( isFileAssigned() ) {
      try {
        switch (oMode) {
        case OPERATIONAL_MODE_PATH:
          if( !isFileRead() )
            readFile();
          result = configFile.toString();
        case OPERATIONAL_MODE_STREAM:
          result = sFile;
        }
      } catch (Exception e) {
        result = e.getMessage();
      }
    }
    return result;
  }

  static public String formatStyle(String message, boolean isError){
    final String MESSAGE_START = "<span class='notice'>"; //mapped to WL's
    // stylesheet
    final String MESSAGE_END = "</span>";
    final String ALERT_START = "<span class='error'>";
    String result = "";
    if( isError )
      result = ALERT_START + message;
    else
      result = MESSAGE_START + message;
    return (result + MESSAGE_END);
  }

  public String processProps(HttpServletRequest request){
    final String HBS = " has been set.<p>";
    final String HBE = " has been encrypted.<p>";

    final String ERROR_WHILE_CHECK = "Error while checking ";
    StringBuffer result = new StringBuffer();
    String nodeName = request.getParameter(ConnectorConfiguratorConstants.PROPERTY_NODENAME.NAME);
    int origDirty = dirtyCount;
    try {
      Connector l = getConnectorByName(nodeName);
      boolean ckOn = request.getParameter(ConnectorConfiguratorConstants.PROPERTY_ISENABLED.NAME) != null;
      if( ckOn != l.getEnabled() ) {
        l.setEnabled(ckOn);
        ++dirtyCount;
        result.append(formatStyle(ConnectorConfiguratorConstants.PROPERTY_ISENABLED.CAPTION + HBS,
            false));
      }

      if( l.getEnabled() ) { //if not enabled, the rest of controls are
        // disabled and don't matter
        String sValue = request.getParameter(ConnectorConfiguratorConstants.PROPERTY_IP.NAME);
        if( !sValue.equals(l.getIp()) ) {
          l.setIp(sValue);
          ++dirtyCount;
          result
              .append(formatStyle(ConnectorConfiguratorConstants.PROPERTY_IP.CAPTION + HBS, false));
        }
        try {
          int port = Integer.parseInt(request
              .getParameter(ConnectorConfiguratorConstants.PROPERTY_PORT.NAME));
          if( port != l.getPort() ) {
            l.setPort(port);
            ++dirtyCount;
            result.append(formatStyle(ConnectorConfiguratorConstants.PROPERTY_PORT.CAPTION + HBS,
                false));
          }
        } catch (Exception e) {
          result
              .append(formatStyle(ERROR_WHILE_CHECK
                  + ConnectorConfiguratorConstants.PROPERTY_PORT.CAPTION + ":<p>" + e.getMessage(),
                  true));
        }
        try {
          int psn = Integer.parseInt(request
              .getParameter(ConnectorConfiguratorConstants.PROPERTY_PSN.NAME));
          if( psn != l.getPrimaryStation() ) {
            l.setPrimaryStation(psn);
            ++dirtyCount;
            result.append(formatStyle(ConnectorConfiguratorConstants.PROPERTY_PSN.CAPTION + HBS,
                false));
          }
        } catch (Exception e) {
          result
              .append(formatStyle(ERROR_WHILE_CHECK
                  + ConnectorConfiguratorConstants.PROPERTY_PSN.CAPTION + ":<p>" + e.getMessage(),
                  true));
        }

        sValue = request.getParameter(ConnectorConfiguratorConstants.PROPERTY_ACCESSCODE.NAME);
        if( l.getEncrypted() ) {
          if( !sValue.equals(getDecryptedValue(l.getAccessCode())) ) {
            l.setAccessCode(getEncryptedValue(sValue));
            ++dirtyCount;
            result.append(formatStyle(ConnectorConfiguratorConstants.PROPERTY_ACCESSCODE.CAPTION
                + HBS, false));
          }
          sValue = request.getParameter(ConnectorConfiguratorConstants.PROPERTY_VERIFYCODE.NAME);
          if( !sValue.equals(getDecryptedValue(l.getVerifyCode())) ) {
            l.setVerifyCode(getEncryptedValue(sValue));
            ++dirtyCount;
            result.append(formatStyle(ConnectorConfiguratorConstants.PROPERTY_VERIFYCODE.CAPTION
                + HBS, false));
          }
        } else {
          boolean eitheror = false;
          if (sValue != null){
            l.setAccessCode(getEncryptedValue(sValue));
            ++dirtyCount;
            eitheror = true;
            result.append(formatStyle(
                ConnectorConfiguratorConstants.PROPERTY_ACCESSCODE.CAPTION + HBE, false));
          }
          
          sValue = request.getParameter(ConnectorConfiguratorConstants.PROPERTY_VERIFYCODE.NAME);
          if (sValue != null){
            l.setVerifyCode(getEncryptedValue(sValue));
            ++dirtyCount;
            eitheror = true;
            result.append(formatStyle(
                ConnectorConfiguratorConstants.PROPERTY_VERIFYCODE.CAPTION + HBE, false));
          }
          if(eitheror)
            l.setEncrypted(true);
        }

        ckOn = request.getParameter(ConnectorConfiguratorConstants.PROPERTY_ISTIMEOUT.NAME) != null;
        if( ckOn != l.getAlwaysUseDefaultAsMin() ) {
          l.setAlwaysUseDefaultAsMin(ckOn);
          ++dirtyCount;
          result.append(formatStyle(ConnectorConfiguratorConstants.PROPERTY_ISTIMEOUT.CAPTION + HBS,
              false));
        }
        int timeout = Integer.parseInt(request
            .getParameter(ConnectorConfiguratorConstants.PROPERTY_TIMEOUT.NAME));
        if( timeout != l.getTimeout() ) {
          l.setTimeout(timeout);
          ++dirtyCount;
          result.append(formatStyle(ConnectorConfiguratorConstants.PROPERTY_TIMEOUT.CAPTION + HBS,
              false));
        }
      }
      if( origDirty < dirtyCount || dirtyCount > 0 ) { //unchecked delete
        Calendar c = Calendar.getInstance();
        c.setTimeInMillis(new Date().getTime());
        l.setTimestamp(c);
        try {
          if( oMode == OPERATIONAL_MODE_PATH )
            if( save() )
              result.append(formatStyle("<p>File was successfully saved.",
                  false));
            else
              result.append(formatStyle("<p>File was not saved.", false));
        } catch (VistaLinkResourceException e) {
          result.append(formatStyle(e.getMessage(), true));
        }
      } else
        result.append(formatStyle("<p>No updates to be saved.", false));
    } catch (Exception e) {
      result.append(formatStyle(e.getMessage(), true));
    }
    return result.toString();
  }

  private String getDecryptedValue(String value){
    return (EncrypterRegistry.getInstance(DESPassPhraseEncrypter.class)
        .decrypt(value));
  }

  private String getEncryptedValue(String value){
    return (EncrypterRegistry.getInstance(DESPassPhraseEncrypter.class)
        .encrypt(value));
  }

  public String generateTree(HttpServletRequest request){
    String result = "";
    TreeNode.setPathDelimeter(TREE_DELIMITER);
    try {
      TreeNode root = new TreeNode(getFilePath().length()<64?getFilePath():getFilePath().substring(0,64)+"...");
      root.setIconSrcOpen("images/network_fancy_icon32x32.gif");
      root.setIconSrcClosed("images/network_fancy_icon32x32.gif");
      TreePopup rPop = new TreePopup("rp");
      rPop.addPopupItem("Add Connector", "addListener(popupNode)", null);
      root.setPopup(rPop);
      int linx = getConnectorArray().length;
      for( int inx = 0; inx < linx; inx++ ) {
        Connector l = getConnectorArray()[inx];
        TreeNode child = root.addChild(l.getJndiName(), "NodeSubmitForm",
            "images/socket32x32.jpg");
        TreePopup lPop = new TreePopup("lp");
        lPop.addPopupItem("Add Connector", "addListener(popupNode)", null);
        lPop
            .addPopupItem("Delete Connector", "deleteListener(popupNode)", null);
        child.setPopup(lPop);
      }
      Tree localTree = new Tree(TREE_NAME, root, true);
      return result + "\n" + localTree.renderTree(request);
    } catch (Exception e) {
      return ExceptionUtils.getFullStackTrace(e);
    }
  }

  public String getPrettyXML(){
    String result = null;
    if( isFileRead() )
      result = configFile.toString();
    return result;
  }

  public boolean deleteStation(String lName, String sName)
      throws VistaLinkResourceException{
    boolean result = false;
    StationNumbers sn = getConnectorByName(lName).getStationNumbers();
    String[] sa = sn.getStationArray();
    for( int inx = 0; inx < sa.length; inx++ ) {
      if( sa[inx].equals(sName) ) {
        sn.removeStation(inx);
        result = true;
        break;
      }
    }
    return result;
  }

  public boolean addStation(String lName, String sName)
      throws VistaLinkResourceException{

    StationNumbers sn = getConnectorByName(lName).getStationNumbers();
    if( sn != null ) {
      for( int inx = 0; inx < sn.getStationArray().length; inx++ )
        if( sn.getStationArray(inx).equals(sName) )
          throw new VistaLinkResourceException("Duplicate Station name!");
    } else
      sn = getConnectorByName(lName).addNewStationNumbers();
    int c = sn.getStationArray().length;
    sn.addStation(sName);
    return c < sn.getStationArray().length;
    //configFile.schemaType().getAttributeModel().
  }

  public void createNewFile(String fileName){
    //configFile = ConnectorsDocument.Factory.newInstance();
    setFileAsString(EMPTY_FILE, fileName);
  }

  /**
   * @return Returns the isAutoSave.
   */
  public boolean isAutoSave(){
    return isAutoSave;
  }

  /**
   * @param isAutoSave
   *          The isAutoSave to set.
   */
  public void setAutoSave(boolean isAutoSave){
    this.isAutoSave = isAutoSave;
  }

  public boolean isDirty(){
    return isFileRead() && dirtyCount > 0;
  }
}