package gov.va.med.environment;

/**
 * Environment settings for J2EE server use.
 * 
 * @author Application Modernization - Foundations Team
 * @version 1.5.0.026
 */
public class Environment {

	/**
	 * Returns whether the administrator has configured the J2EE server to be "production" in a VA-medical-center sense,
	 * i.e., is this system operating on production VA data. The source of the setting is the
	 * gov.va.med.environment.production JVM argument passed to the J2EE server upon startup. A setting of true
	 * desginates the server as a production server; any other value (including not passing the JVM argument at all)
	 * marks the server as not a VA production server.
	 * @return true if the server is a VA production server, false if not
	 */
	public static boolean isProduction() {

		boolean returnVal = Boolean.getBoolean("gov.va.med.environment.production");
		return returnVal;
	}

	/**
	 * Returns the J2EE server type. The source of the setting is the gov.va.med.environment.servertype JVM argument
	 * passed to the J2EE server upon startup. Defaults to return UNKNOWN if the JVM argument is not present.
	 * @return ServerType: JBOSS | ORACLE | SUN_RI_13 | UNKNOWN | WEBLOGIC | JAVASE | WEBSPHERE
	 */
	public static ServerType getServerType() {

		ServerType returnVal = null;
		String serverProperty = System.getProperty("gov.va.med.environment.servertype");
		if (serverProperty != null) {
			returnVal = ServerType.getServerTypeObject(serverProperty.toLowerCase());
		}
		if (returnVal == null) {
			// so we don't return null
			returnVal = ServerType.UNKNOWN;
		}
		return returnVal;
	}
}