package gov.va.med.vistalink.adapter.cci;

import java.util.ArrayList;

import javax.resource.cci.ConnectionSpec;

import org.w3c.dom.Document;
import org.w3c.dom.Node;

/**
 * This interface defined the common properties needed by any connection spec
 * implementation.
 * <p>
 * It identifies the most common property(s): Currently, division.
 * <p>
 * The sub-classes should extend if there are more properties.
 * 
 * @author Application Modernization - Foundations Team
 * @version 1.5.0.026
 */
public interface VistaLinkConnectionSpec extends ConnectionSpec {
	/**
	 * Returns the user name of this ConnectionSpec.
	 * 
	 * @return the division, in String
	 */
	public String getDivision();

	/**
	 * Sets the division
	 * 
	 * @param division
	 *            the division number to be set
	 */
	public void setDivision(String division);

	/**
	 * creates the xml in the security node
	 * 
	 * @param requestDoc
	 *            the Document object that contains the nodes
	 * @param securityNode
	 *            the node to create the XML under
	 * @va.exclude
	 */
	public void setAuthenticationNodes(Document requestDoc, Node securityNode);

	/**
	 * clears the information in the security node
	 * 
	 * @param securityNode
	 *            the node to clear
	 * @va.exclude
	 */
	public void clearSecurityNode(Node securityNode);

	/**
	 * Gets the security information as a proprietary string
	 * 
	 * @param securityNode
	 *            the node to get the information from
	 * @return Security info
	 * @va.exclude
	 */
	public ArrayList getProprietarySecurityInfo();

	/**
	 * Compares two objects to see if they are equal
	 * @param obj
	 *            the object to compare
	 * @va.exclude
	 * @deprecated
	 */
	public boolean isConnSpecEqual(Object obj);

	/**
	 * returns the security mechanism type
	 * 
	 * @param obj
	 *            the object to compare
	 * @va.exclude
	 */
	public String getSecurityType();

}