package gov.va.med.vistalink.adapter.record;

import gov.va.med.exception.FoundationsException;

/**
 * Base request interface
 * 
 * @author Application Modernization - Foundations Team
 * @version 1.5.0.026
 */
public interface VistaLinkRequestVO {

	/**
	 * Returns request string that is written to the socket sent to MUMPS.
	 * 
	 * @return String represents the request to M
	 * @throws FoundationsException
	 * @va.exclude
	 */
	public String getRequestString() throws FoundationsException;

	/**
	 * Returns current connection time out value
	 * 
	 * @return int Time out value in milli-seconds
	 */
	public int getTimeOut();

	/**
	 * Enables application to set time out for read operations on connections.
	 * 
	 * @param timeOut
	 *            Time out value to set in milli-seconds. This timeout value is
	 *            compared to the default value usually used for the connection.
	 *            The greater of the two values will be used.
	 */
	public void setTimeOut(int timeOut);
	
	/**
	 * Returns current retry strategy reference
	 * 
	 * @return VistaLinkRequestRetryStrategy instance reference
	 */
	public VistaLinkRequestRetryStrategy getRetryStrategy();
	
	/**
	 * Enables application to set retry strategy in request executed failed because of socket
	 * failure or other system type problems. The strategy detemines if the retry should
	 * be attempted.
	 * 
	 * @param VistaLinkRequestRetryStrategy instance representing the strategy the application wants implemented.
	 */
	public void setRetryStrategy(VistaLinkRequestRetryStrategy strategy);

}