package gov.va.med.vistalink.adapter.spi;

import java.util.Map;

import javax.management.ObjectName;

/**
 * MBean interface for the main VistaLink MBean. It used by the VistaLink console to provide information on VistaLink
 * deployed connectors, using the WebLogic MBean hierarchy to obtain the list of WebLogic-deployed connectors.
 * 
 * @author Application Modernization - Foundations Team
 * @version 1.5.0.026
 */
public interface VistaLinkRuntimeInfoMBean {

	/**
	 * Retrieve a list (Map) of VistaLink deployed (partially or fully) connectors.
	 * @return Map. The key is the canonical MBean object name of the ConnectorComponentRuntimeMBean representing each
	 *         connector. The value is a ConnectorInfoVO object with summary fields populated, for the partial (less
	 *         than detailed) information about the connector.
	 * @va.exclude
	 */
	public Map retrieveConnectorList();

	/**
	 * Retrieve detailed information for a particular VistaLink deployed (partially or fully) connector.
	 * @param canonicalRuntimeMBeanObjectName canonical MBean object name of the ConnectorComponentRuntimeMBean
	 *            representing the connector to retrieve information for.
	 * @return ConnectorInfoVO object with detailed information about the specified connector
	 * @va.exclude
	 */
	public ConnectorInfoVO retrieveConnectorDetailInfo(String canonicalRuntimeMBeanObjectName);
	
	/**
	 * 
	 * @return the ObjectName under which the MBean was registered.
	 * @va.exclude
	 */
	public ObjectName getObjectName();
}