/*
 * Created on Oct 20, 2003
 *
 */
package gov.va.med.vistalink.adapter.spi;

import gov.va.med.exception.ExceptionUtils;
import gov.va.med.exception.FoundationsException;
import gov.va.med.vistalink.adapter.record.VistaLinkRequestVO;
import gov.va.med.vistalink.adapter.record.VistaLinkResponseFactoryImpl;
import gov.va.med.vistalink.adapter.record.VistaLinkResponseVO;

import java.util.HashMap;

import org.apache.log4j.Logger;
import org.apache.log4j.Priority;
import org.jaxen.JaxenException;
import org.jaxen.XPath;
import org.jaxen.dom.DOMXPath;
import org.w3c.dom.Attr;
import org.w3c.dom.Document;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

/**
 * 
 * @author Application Modernization - Foundations Team
 * @version 1.5.0.026
 */
public class VistaLinkSystemInfoResponseFactory extends VistaLinkResponseFactoryImpl {

	/**
	 * The logger used for this class
	 */
	private static final Logger logger = Logger.getLogger(VistaLinkSystemInfoResponseFactory.class);

	/**
	 *  
	 */
	public VistaLinkSystemInfoResponseFactory() {
		super();
	}

	/**
	 * Evaluates the response from M and gets the heart beat rate in millis.
	 * Constructs a new VistaLinkSystemInfoResponse object.
	 * 
	 * @see gov.va.med.vistalink.adapter.record.VistaLinkResponseFactoryImpl#parseMessageBody(java.lang.String,
	 *      java.lang.String, org.w3c.dom.Document, java.lang.String,
	 *      gov.va.med.vistalink.adapter.record.VistaLinkRequestVO)
	 */
	protected VistaLinkResponseVO parseMessageBody(String rawXml, String filteredXml, Document doc, String messageType,
			VistaLinkRequestVO requestVO) throws FoundationsException {

		try {
			XPath xpath = new DOMXPath("/VistaLink/Response/SystemInfo/.");
			Node resultsNode = (Node) xpath.selectSingleNode(doc);
			// get attribute map
			NamedNodeMap attrs = resultsNode.getAttributes();
			final String deflt = "unknown";
			HashMap hm = new HashMap();
			// get individual attribute values
			hm.put("vistalinkVersion", getAttributeValue(attrs, "vistalinkVersion", deflt));
			hm.put("vistalinkBuild", getAttributeValue(attrs, "vistalinkBuild", deflt));
			hm.put("appServerTimeout", getAttributeValue(attrs, "appServerTimeout", "0"));
			hm.put("reAuthSessionTimeout", getAttributeValue(attrs, "reAuthSessionTimeout", "0"));
			hm.put("uci", getAttributeValue(attrs, "uci", deflt));
			hm.put("vol", getAttributeValue(attrs, "vol", deflt));
			hm.put("boxVolume", getAttributeValue(attrs, "boxVolume", deflt));
			hm.put("mVersion", getAttributeValue(attrs, "mVersion", deflt));
			hm.put("operatingSystem", getAttributeValue(attrs, "operatingSystem", deflt));
			hm.put("domainName", getAttributeValue(attrs, "domainName", deflt));
			hm.put("vistaProduction", getAttributeValue(attrs, "vistaProduction", deflt));
			hm.put("defaultInstitution", getAttributeValue(attrs, "defaultInstitution", deflt));
			hm.put("cpName", getAttributeValue(attrs, "cpName", deflt));
			
			NodeList introNodeList = doc.getDocumentElement().getElementsByTagName("IntroText");
			if (introNodeList.getLength() == 1) {
				NodeList cdataNodeList = introNodeList.item(0).getChildNodes();
				if ((cdataNodeList.getLength() == 1) && (cdataNodeList.item(0).getNodeType() == Node.CDATA_SECTION_NODE)) {
					hm.put("introductoryText", cdataNodeList.item(0).getNodeValue());
				}
			}

			if (logger.isDebugEnabled()) {
				logger.debug("Retrieved system information response attributes");
			}
			// create response object
			return new VistaLinkSystemInfoResponse(rawXml, filteredXml, doc, messageType, hm);
		} catch (JaxenException e) {
			String errStr = "could not parse xml";
			if (logger.isEnabledFor(Priority.ERROR)) {
				String errMsg = (new StringBuffer()).append(errStr).append("\n\t").append(
						ExceptionUtils.getFullStackTrace(e)).toString();
				logger.error(errMsg);
			}
			throw new FoundationsException(errStr, e);
		}
	}

	private String getAttributeValue(NamedNodeMap attrs, String attrName, String deflt) {
		Attr attr = null;
		attr = (Attr) attrs.getNamedItem(attrName);
		return (attr != null) ? attr.getValue() : deflt;
	}
}