/*
 * Created on May 18, 2005
 *
 */
package gov.va.med.vistalink.cache;

import gov.va.med.vistalink.adapter.cci.VistaLinkConnectionFactory;

import java.util.Collections;
import java.util.Map;

import javax.naming.InitialContext;
import javax.naming.Context;
import javax.naming.NamingException;

import org.apache.log4j.Logger;

/**
 * @author Application Modernization - Foundations Team
 *
 * @version 1.5.0.026
 */
public class VistaLinkConnectionFactoryLocator{

  private static VistaLinkConnectionFactoryLocator me;
  Context context;
  Map map = Collections.synchronizedMap(new WeakValueMap());
  Logger logger = Logger.getLogger(VistaLinkConnectionFactoryLocator.class); 
    
  private VistaLinkConnectionFactoryLocator() throws VistaLinkCacheException {
  }  
  
  public static VistaLinkConnectionFactoryLocator getInstance() throws VistaLinkCacheException {
    if (me == null) {
      me = new VistaLinkConnectionFactoryLocator();
    }
    return me;
  }
  
  public VistaLinkConnectionFactory getFactory(String jndiName) 
                                    throws VistaLinkConnectionFactoryNotFoundException,
                                           VistaLinkCacheException {
    if(context == null){
      logger.debug("Initial context is unspecified, creating InitialContext");
      try{
        setContext(new InitialContext());
      }
      catch(NamingException e){
        throw new VistaLinkCacheException("Unable to create InitialContext, "+ e.getMessage());
      }
    }
    VistaLinkConnectionFactory result = (VistaLinkConnectionFactory)map.get(jndiName);
    if(result == null)
      try {
        result = (VistaLinkConnectionFactory)context.lookup(jndiName);
        if(result != null)
          map.put(jndiName, result);
        else
          throw new VistaLinkConnectionFactoryNotFoundException("Unable to find ConnectionFactory, based on the jndiName provided - " + jndiName);
      }
      catch(NamingException e){
        throw new VistaLinkCacheException("Could not complete the lookup: " + e.getMessage());
      }
    return result;
  }
  
  public void setContext(Context ctx){ //Use for Cactus testing so far
    context = ctx;
  }
}
