package gov.va.med.vistalink.institution;

import java.util.Iterator;
import java.util.Map;
import java.util.Set;

import javax.management.InstanceAlreadyExistsException;
import javax.management.MBeanRegistrationException;
import javax.management.MBeanServer;
import javax.management.MalformedObjectNameException;
import javax.management.NotCompliantMBeanException;
import javax.management.ObjectInstance;
import javax.management.ObjectName;
import javax.naming.InitialContext;
import javax.naming.NamingException;

import org.apache.log4j.Logger;

import weblogic.management.MBeanHome;
import weblogic.management.runtime.ServerRuntimeMBean;

/**
 * @author Application Modernization - Foundations Team
 * @version 1.5.0.026
 */
public class InstitutionMapManager implements InstitutionMapManagerMBean {

	private static final Logger logger = Logger.getLogger(InstitutionMapManager.class);

	public static void register() {

		logger.debug("\nInstitutionMapManager :: In startup().");
		try {
			InitialContext ctx = new InitialContext();
			/*
			 * Note: we get this rather than MBeanHome because we're interacting with non-WebLogic Server MBeans.
			 */
			MBeanHome home = (MBeanHome) ctx.lookup(MBeanHome.LOCAL_JNDI_NAME);
			String domainName = home.getDomainName();
			String serverName = "";
			logger.debug("Got MBeanHome for " + MBeanHome.LOCAL_JNDI_NAME + ".");
			MBeanServer mbeanServer = home.getMBeanServer();
			// register bean for institution mapping
			Set serverMBeans = home.getMBeansByType("ServerRuntime");
			Iterator iter = serverMBeans.iterator();
			while (iter.hasNext()) {
				// using LOCAL_JNDI_NAME, should only be one
				serverName = ((ServerRuntimeMBean) iter.next()).getName();
				break;
			}
			ObjectName mbeanName = new ObjectName(getServerSpecificCanonicalObjectName(domainName, serverName));
			InstitutionMapManager mapManager = new InstitutionMapManager();
			ObjectInstance mbeanInstance = mbeanServer.registerMBean((Object) mapManager, mbeanName);
			logger.debug("\nInstitutionMapManager :: InstitutionMapManager MBean registered....");

		} catch (NamingException e) {
			logger.error("\nInstitutionMapManager :: MBean registration failed: ", e);
		} catch (MalformedObjectNameException e) {
			logger.error("\nInstitutionMapManager :: MBean registration failed: ", e);
		} catch (InstanceAlreadyExistsException e) {
			// this one is not necessarily an ERROR, someone is just trying
			// to reload bean but it's already loaded
			logger.debug("\nInstitutionMapManager :: MBean registration failed, already loaded.");
		} catch (NotCompliantMBeanException e) {
			logger.error("\nInstitutionMapManager :: MBean registration failed: ", e);
		} catch (MBeanRegistrationException e) {
			logger.error("\nInstitutionMapManager :: MBean registration failed: ", e);
		}
	}

	public InstitutionMapManager() {
	}

	public Map getMappings() {
		// return value must be serializable for remote MBean use
		Map returnVal = null;
		InstitutionMappingImpl me = InstitutionMappingImpl.getInstance();
		if (me != null) {
			returnVal = me.getMappingClone();
		}
		return returnVal;
	}

	/**
	 * Utility method for users outside this class, to create a canonical object name to retrieve an instance of this
	 * MBean on a given server. Also used internally to create the ObjectName the runtime object is registered under.
	 * @param domainName domain to use in naming of MBean
	 * @param serverName server name to use in naming of MBean
	 * @return canonical object name string for use in constructing ObjectName for MBean
	 * @va.exclude
	 */
	public static String getServerSpecificCanonicalObjectName(String domainName, String serverName) {

		StringBuffer sb = new StringBuffer(domainName).append(":Name=").append(serverName).append(
				",Type=InstitutionMapManager,Location=").append(serverName);
		return sb.toString();
	}
}