package gov.va.med.vistalink.institution;

import java.util.Set;

/**
 * Interface for cross-classloader access to institution mapping singleton object.
 * <p>
 * The singleton instance of this class is initialized during connector startup.
 * 
 * @author Application Modernization - Foundations Team
 * @version 1.5.0.026
 * @va.exclude
 */
public interface InstitutionMapping {
	/**
	 * Returns the JNDI connector name (if found) for a division matching the station number passed in.
	 * 
	 * @param stationNumber
	 *            institution station number for Vista Institution
	 * @return Jndi Connector name, if a connector is found that is mapped to input station number. Use for a JNDI
	 *         lookup of actual VistaLink connector.
	 * @throws InstitutionMapNotInitializedException
	 *             thrown if the institution mapping has not been initialized
	 * @throws InstitutionMappingNotFoundException
	 *             thrown if no connector JNDI name has been mapped to input station number
	 * @va.exclude
	 */
	public abstract String getJndiConnectorNameForInstitution(String stationNumber)
			throws InstitutionMapNotInitializedException, InstitutionMappingNotFoundException;

	/**
	 * Returns the set of station# strings containing the current set of registered institution mappings.
	 * 
	 * @return Set containing station# strings.
	 * @throws InstitutionMapNotInitializedException
	 */
	public abstract Set getVistaLinkMappedStationNumberSet() throws InstitutionMapNotInitializedException;
}