package gov.va.med.vistalink.institution;

/**
 * Factory used by the InstitutionMappingDelegate class to get access to the
 * mapping singleton object.
 * <p>
 * The in-memory mapping is initialized during connector startup.
 * 
 * @author Application Modernization - Foundations Team
 * @version 1.5.0.026
 */
public class InstitutionMappingFactory {

	private static InstitutionMappingImpl singleton = null;

	/**
	 * The Singleton instance self-registers here when it creates itself. Its
	 * creation is synchronized, so it's not possible that two singletons would
	 * contend to register themselves in this method.
	 * 
	 * @param singletonInstance
	 * @va.exclude
	 */
	public static void registerSingleton(InstitutionMappingImpl singletonInstance) {
		singleton = singletonInstance;
	}

	/**
	 * return the institution mapping singleton instance to the business
	 * delegate.
	 * 
	 * @return @va.exclude
	 */
	public static InstitutionMapping getInstitutionMapping() {
		return singleton;
	}

}