package gov.va.med.vistalink.samples;

import gov.va.med.exception.ExceptionUtils;
import gov.va.med.exception.FoundationsException;
import gov.va.med.vistalink.adapter.cci.VistaLinkAppProxyConnectionSpec;
import gov.va.med.vistalink.adapter.cci.VistaLinkCcowConnectionSpec;
import gov.va.med.vistalink.adapter.cci.VistaLinkConnection;
import gov.va.med.vistalink.adapter.cci.VistaLinkConnectionFactory;
import gov.va.med.vistalink.adapter.cci.VistaLinkConnectionSpec;
import gov.va.med.vistalink.adapter.cci.VistaLinkDuzConnectionSpec;
import gov.va.med.vistalink.adapter.cci.VistaLinkVpidConnectionSpec;
import gov.va.med.vistalink.adapter.record.VistaLinkFaultException;
import gov.va.med.vistalink.institution.InstitutionMappingDelegate;
import gov.va.med.vistalink.rpc.RpcRequest;
import gov.va.med.vistalink.rpc.RpcRequestFactory;
import gov.va.med.vistalink.rpc.RpcResponse;

import java.io.IOException;

import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.NamingException;
import javax.resource.ResourceException;
import javax.servlet.RequestDispatcher;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.log4j.Logger;

/**
 * Controller servlet for MVC-architected sample web application.
 * <p>
 * Demonstrates how to use VistaLink in a J2EE environment to execute RPCs and retrieve data from M.
 * 
 * @author Application Modernization - Foundations Team
 * @version 1.5.0.026
 */
public class VistaLinkJ2EESample extends HttpServlet {

	private static final Logger logger = Logger.getLogger(VistaLinkJ2EESample.class);

	/**
	 * @va.exclude
	 */
	public void doPost(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException {
		doGetAndPost(request, response);
	}

	/**
	 * @va.exclude
	 */
	public void doGet(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException {
		doGetAndPost(request, response);
	}

	private void doGetAndPost(HttpServletRequest request, HttpServletResponse response) throws ServletException,
			IOException {

		VistaLinkConnectionSpec connSpec = null;
		StringBuffer results = new StringBuffer();

		// collect form parameters
		// TODO run through transform to protect against bad submissions?
		String page = request.getParameter("page");
		String division = request.getParameter("division");
		String duz = request.getParameter("duz");
		String vpid = request.getParameter("vpid");
		String ccowHandle = request.getParameter("ccowHandle");
		String appProxyName = request.getParameter("appProxyName");
		boolean connRefLookup = "lookup".equals(request.getParameter("connector"));
		boolean connRefSelect = "select".equals(request.getParameter("connector"));

		if (page != null) {
			RequestDispatcher dispatcher = getServletContext().getRequestDispatcher("/" + page);
			dispatcher.forward(request, response);
			return;
		}

		// create connection spec
		if (vpid != null) {
			connSpec = new VistaLinkVpidConnectionSpec(division, vpid);
			request.setAttribute("division", division);
			request.setAttribute("vpid", vpid);
		} else if (duz != null) {
			connSpec = new VistaLinkDuzConnectionSpec(division, duz);
			request.setAttribute("division", division);
			request.setAttribute("duz", duz);
		} else if (appProxyName != null) {
			connSpec = new VistaLinkAppProxyConnectionSpec(division, appProxyName);
			request.setAttribute("division", division);
			request.setAttribute("appProxyName", appProxyName);
		} else if (ccowHandle != null) {
			connSpec = new VistaLinkCcowConnectionSpec(ccowHandle);
			request.setAttribute("ccowHandle", ccowHandle);
		} else {
			RequestDispatcher dispatcher = getServletContext().getRequestDispatcher("/LoginChoice.jsp");
			dispatcher.forward(request, response);
			return;
		}

		Context ic = null;
		String jndiName = "";
		VistaLinkConnectionFactory cf = null;
		VistaLinkConnection myConnection = null;
		RpcRequest vReq = null;
		RpcResponse vResp = null;
		String rpcName = "";
		// rpc success/failure indicator
		request.setAttribute("success", "false");

		if (connSpec != null) {

			try {
				ic = new InitialContext();
				if (connRefLookup) {
					jndiName = InstitutionMappingDelegate.getJndiConnectorNameForInstitution(division);
				} else if (connRefSelect) {
					jndiName = request.getParameter("connectorChoice");
				} else {
					// note, a resource-ref mapping is used to create the resource
					// reference used below
					jndiName = "eis/vlj/testconnector";
					// since we're using an application-specific resource-ref, need to 
					// change context to the application's ENC (environment naming context)
					ic = (Context) ic.lookup("java:comp/env/");
				}
				request.setAttribute("jndiName", jndiName);
				cf = (VistaLinkConnectionFactory) ic.lookup(jndiName);
				myConnection = (VistaLinkConnection) cf.getConnection(connSpec);

				rpcName = "XOBV TEST PING";
				vReq = RpcRequestFactory.getRpcRequest();
				// set request timeout to twice connection's normal value
				vReq.setTimeOut(myConnection.getTimeOut() * 2);
				vReq.setUseProprietaryMessageFormat(true);
				vReq.setRpcContext("XOBV VISTALINK TESTER");
				vReq.setRpcName(rpcName);
				vResp = myConnection.executeRPC(vReq);
				results.append(rpcName + " Results: \n" + vResp.getResults() + "\n");

				rpcName = "XOBV TEST WORD PROCESSING";
				vReq.setRpcName(rpcName);
				vReq.clearParams();
				vResp = myConnection.executeRPC(vReq);
				results.append("\n" + rpcName + " Results: \n" + vResp.getResults() + "\n");

				rpcName = "XOBV TEST STRING";
				vReq.setRpcName(rpcName);
				vReq.clearParams();
				vReq.getParams().setParam(1, "string", "This is a test string!");
				vResp = myConnection.executeRPC(vReq);
				results.append("\n" + rpcName + " Results: \n" + vResp.getResults() + "\n");

				rpcName = "XOBV TEST GLOBAL NODE";
				vReq.setRpcName(rpcName);
				vReq.clearParams();
				vReq.getParams().setParam(1, "string", "Hello World");
				vResp = myConnection.executeRPC(vReq);
				results.append("\n" + rpcName + " Results: \n" + vResp.getResults() + "\n");

				rpcName = "XOBV TEST RPC LIST";
				vReq.setRpcName(rpcName);
				vReq.clearParams();
				vReq.getParams().setParam(1, "string", "XOB");
				vResp = myConnection.executeRPC(vReq);
				results.append("\n" + rpcName + " Results: \n" + vResp.getResults());

				logger.debug("got results, length: '" + results.length() + "'");

				request.setAttribute("success", "true");

			} catch (VistaLinkFaultException e) {

				logger.error("Exception occurred executing " + rpcName + "\nException:", e);
				results.append("Exception occurred executing " + rpcName + "\nException:"
						+ ExceptionUtils.getFullStackTrace(e));

			} catch (NamingException e) {

				logger.error("Exception occurred executing " + rpcName + "\nException:", e);
				results.append("Exception occurred:\n&nbsp;\n" + e.getMessage() + "\n&nbsp;\n"
						+ ExceptionUtils.getFullStackTrace(e));

			} catch (ResourceException e) {

				logger.error("Exception occurred executing " + rpcName + "\nException:", e);
				results.append("Exception occurred:\n&nbsp;\n" + e.getMessage() + "\n&nbsp;\n"
						+ ExceptionUtils.getFullStackTrace(e));

			} catch (FoundationsException e) {

				logger.error("Exception occurred executing " + rpcName + "\nException:", e);
				results.append("Exception occurred executing " + rpcName + "\nException:"
						+ ExceptionUtils.getFullStackTrace(e));

			} finally {

				try {
					ic.close();
				} catch (NamingException e) {
					logger.error("Error closing context: ", e);
				}
				if (myConnection != null) {
					try {
						myConnection.close();
					} catch (ResourceException e) {
						logger.error("Error closing connection:", e);
					}
				}
			}
		}

		// display results
		request.setAttribute("results", results.toString());
		RequestDispatcher dispatcher = getServletContext().getRequestDispatcher("/DisplayResults.jsp");
		dispatcher.forward(request, response);
		return;

	}
}