package gov.va.med.vistalink.samples;

import gov.va.med.vistalink.adapter.spi.ConnectorInfoVO;
import gov.va.med.vistalink.adapter.spi.VistaLinkRuntimeInfo;

import java.util.Iterator;
import java.util.Map;
import java.util.Set;
import java.util.TreeSet;

import javax.management.InstanceNotFoundException;
import javax.management.MBeanServer;
import javax.management.ObjectName;
import javax.naming.InitialContext;

import org.apache.log4j.Logger;

import weblogic.management.MBeanHome;
import weblogic.management.runtime.ServerRuntimeMBean;

/**
 * This class uses "unsupported" (non-public) API calls in VistALink, as well as WebLogic-specific calls, to display a
 * drop-down list of the currently deployed VistALink connectors. This is to make the sample application more useful as
 * a "testing" application to test the viability of deployed connectors.
 * 
 * @author Application Modernization - Foundations Team
 * @version 1.5.0.026
 */
public class ZConnectorListHelper {

	static Logger logger = Logger.getLogger(ZConnectorListHelper.class);

	public Set getConnectorJndiNameList() throws Exception {

		String domainName = null;
		TreeSet returnVal = new TreeSet();
		Map connectorMap = null;

		try {

			InitialContext ctx = new InitialContext();
			MBeanHome home = (MBeanHome) ctx.lookup(MBeanHome.LOCAL_JNDI_NAME);
			MBeanServer localMBeanServer = home.getMBeanServer();
			domainName = home.getDomainName();

			String canonicalVljMcfMonitorName = VistaLinkRuntimeInfo.getServerSpecificCanonicalObjectName(domainName,
					getServerName(home));
			logger.debug("about to make remote JMX retrieveConnectorList call.");
			connectorMap = (Map) localMBeanServer.invoke(new ObjectName(canonicalVljMcfMonitorName),
					"retrieveConnectorList", new Object[0], new String[0]);
			if (logger.isDebugEnabled()) {
				logger.debug("Returned with connector list with " + connectorMap.size() + " entries.");
			}
		} catch (InstanceNotFoundException e) {
			// means no connectors have been deployed...?
			logger.debug(e);
			throw new Exception(
					"Could not find VistaLinkRunTimeInfo MBean. This may mean no VistaLink connectors have been deployed.");
		} catch (Exception e) {
			logger.error(e);
			throw e;
		}
		if (connectorMap != null) {
			Iterator iter = connectorMap.keySet().iterator();
			while (iter.hasNext()) {
				String key = (String) iter.next();
				logger.debug("found key: " + key);
				ConnectorInfoVO connVO = (ConnectorInfoVO) connectorMap.get(key);
				returnVal.add(connVO.getWlsJndiActual());
			}
		}
		return returnVal;
	}

	private static String getServerName(MBeanHome localHome) {
		String returnVal = null;
		Set serverMBeans = localHome.getMBeansByType("ServerRuntime");
		Iterator iter = serverMBeans.iterator();
		while (iter.hasNext()) {
			// using local home, should be only one
			returnVal = ((ServerRuntimeMBean) iter.next()).getName();
			break;
		}
		return returnVal;
	}

}